<?php

###################################################
# 読み込み
###################################################

require_once(get_template_directory().'/include/classes/Toc_Shortcode.php');

###################################################
# 定数
###################################################

// テーマのバージョン
const AFZ_VERSION = '1.0.0';

// テーマのディレクトリ
const AFZ_THEME_DIRECTORY = __DIR__;

// テーマの設定ページのスラッグ
const AFZ_THEME_SETTING_PAGE_SLUG = 'afeez_settings_page';

// テーマの設定ページのタイトル
const AFZ_THEME_SETTINGS_PAGE_TITLE = 'Afeez Edge';

// テーマ設定のオプショングループ
const AFZ_THEME_SETTINGS_OPTION_GROUP = 'afeez_settings';

// メニューの表示位置
const AFZ_THEME_SETTINGS_MENU_POSITION = 2.99;

// 抜粋する文字数
const AFZ_EXCERPT_LENGTH = 80;

// 抜粋末尾に表示する文字
const AFZ_EXCERPT_END_TEXT = '...';

###################################################
# アクション
###################################################

add_action('after_switch_theme', 'afz_theme_install');
add_action('switch_theme', 'afz_theme_uninstall');

add_action('after_setup_theme', 'afz_add_theme_support');
add_action('after_setup_theme', 'afz_add_post_type_support');
add_action('after_setup_theme', 'afz_register_navigation_menus');
add_action('after_setup_theme', 'afz_set_post_thumbnail_size');
add_action('after_setup_theme', 'afz_add_image_size');

add_action('admin_init', 'afz_register_theme_settings');
add_action('admin_menu', 'afz_add_theme_menu_page');

###################################################
# フィルター
###################################################

add_filter('excerpt_more', 'afz_change_excerpt_end_text');
add_filter('excerpt_length', 'afz_change_excerpt_length');

###################################################
# ショートコード
###################################################

new Toc_Shortcode();

add_shortcode('ranking', 'afz_shortcode_ranking');
add_shortcode('balloon', 'afz_shortcode_balloon');
add_shortcode('icon_balloon_left', 'afz_shortcode_icon_balloon_left');
add_shortcode('icon_balloon_right', 'afz_shortcode_icon_balloon_right');
add_shortcode('marker', 'afz_short_code_marker');
add_shortcode('box', 'afz_short_code_box');

###################################################
# 関数
###################################################

/**
 * テーマのインストール
 */
function afz_theme_install()
{
    if (get_option('afz_theme_color') === false) {
        add_option('afz_theme_color', 'afz_theme_pink');
    }

    if (get_option('afz_logo_url') === false) {
        add_option('afz_logo_url', '');
    }

    if (get_option('afz_profile_name') === false) {
        add_option('afz_profile_name', '匿名');
    }

    if (get_option('afz_profile_content') === false) {
        add_option('afz_profile_content', 'プロフィールが設定されていません。');
    }

    if (get_option('afz_profile_image_url') === false) {
        add_option('afz_profile_image_url', get_template_directory_uri().'/assets/img/no_image.png');
    }

    if (get_option('afz_profile_twitter_id') === false) {
        add_option('afz_profile_twitter_id', '');
    }

    if (get_option('afz_profile_instagram_id') === false) {
        add_option('afz_profile_instagram_id', '');
    }

    if (get_option('afz_profile_facebook_id') === false) {
        add_option('afz_profile_facebook_id', '');
    }
}

/**
 * テーマのアンインストール
 */

function afz_theme_uninstall()
{
    delete_option('afz_theme_color');
    delete_option('afz_logo_url');
    delete_option('afz_profile_name');
    delete_option('afz_profile_content');
    delete_option('afz_profile_image_url');
    delete_option('afz_profile_twitter_id');
    delete_option('afz_profile_instagram_id');
    delete_option('afz_profile_facebook_id');
}

/**
 * テーマ機能を有効化する
 */

function afz_add_theme_support()
{
    add_theme_support('post-thumbnails');
    add_theme_support('menus');
}

/**
 * テーマの設定項目を有効化する
 */

function afz_register_theme_settings()
{
    register_setting(AFZ_THEME_SETTINGS_OPTION_GROUP, 'afz_theme_color');
    register_setting(AFZ_THEME_SETTINGS_OPTION_GROUP, 'afz_logo_url');
    register_setting(AFZ_THEME_SETTINGS_OPTION_GROUP, 'afz_profile_name');
    register_setting(AFZ_THEME_SETTINGS_OPTION_GROUP, 'afz_profile_content');
    register_setting(AFZ_THEME_SETTINGS_OPTION_GROUP, 'afz_profile_image_url');
    register_setting(AFZ_THEME_SETTINGS_OPTION_GROUP, 'afz_profile_twitter_id');
    register_setting(AFZ_THEME_SETTINGS_OPTION_GROUP, 'afz_profile_instagram_id');
    register_setting(AFZ_THEME_SETTINGS_OPTION_GROUP, 'afz_profile_facebook_id');
}

/**
 * テーマのメニューを追加する
 */

function afz_add_theme_menu_page()
{
    add_menu_page(AFZ_THEME_SETTINGS_PAGE_TITLE, AFZ_THEME_SETTINGS_PAGE_TITLE, 'edit_themes', AFZ_THEME_SETTING_PAGE_SLUG, 'afz_settings_page', '', AFZ_THEME_SETTINGS_MENU_POSITION);
}

/**
 * テーマの設定ページを出力する
 */

function afz_settings_page()
{
    include AFZ_THEME_DIRECTORY.'/pages/settings.php';
}

/**
 * テーマカラーのクラス名を取得する
 *
 * @return bool|mixed|void
 */

function afz_get_theme_color()
{
    return get_option('afz_theme_color', '');
}

/**
 * ロゴのURLを取得する
 *
 * @return bool|mixed|void
 */

function afz_get_logo_url()
{
    return get_option('afz_logo_url', '');
}

/**
 * プロフィールの名前を取得する
 *
 * @return bool|mixed|void
 */

function afz_get_profile_name()
{
    return get_option('afz_profile_name', '');
}

/**
 * プロフィールの説明を取得する
 *
 * @return bool|mixed|void
 */

function afz_get_profile_content()
{
    return get_option('afz_profile_content', '');
}

/**
 * プロフィールの画像URLを取得する
 *
 * @return bool|mixed|void
 */

function afz_get_profile_image_url()
{
    return get_option('afz_profile_image_url', '');
}

/**
 * TwitterのIDを取得する
 *
 * @return bool|mixed|void
 */

function afz_get_profile_twitter_id()
{
    return get_option('afz_profile_twitter_id', '');
}

/**
 * InstagramのIDを取得する
 *
 * @return bool|mixed|void
 */

function afz_get_profile_instagram_id()
{
    return get_option('afz_profile_instagram_id', '');
}

/**
 * FacebookのIDを取得する
 *
 * @return bool|mixed|void
 */

function afz_get_profile_facebook_id()
{
    return get_option('afz_profile_facebook_id', '');
}

/**
 * 固定ページの抜粋を有効化する
 */

function afz_add_post_type_support()
{
    add_post_type_support('page', 'excerpt');
}

/**
 * ナビゲーションメニューを登録する
 */

function afz_register_navigation_menus()
{
    register_nav_menus([
        'header' => __('ヘッダー'),
    ]);
}

/**
 * サムネイルのサイズを指定する
 */

function afz_set_post_thumbnail_size()
{
    set_post_thumbnail_size(600, 600, true);
}

/**
 * 自動リサイズを行う際のサイズを追加する
 */

function afz_add_image_size()
{
    add_image_size('ogp', 1200, 630, false);
    add_image_size('ogp_cropped', 1200, 630, true);
}

/**
 * ページのタイトルを返す
 *
 * @return string|void
 */

function afz_get_title()
{
    $blog_name = get_bloginfo('name');

    if (is_home() || is_front_page()) {
        return $blog_name;
    }

    return wp_title(' | ', true, 'right').$blog_name;
}

/**
 * 現在URLを返す
 *
 * @return string|void
 */

function afz_current_url()
{
    global $wp;

    return home_url(add_query_arg([$_GET], $wp->request));
}

/**
 * 関連記事を取得するためのクエリを返す
 *
 * @param $post_id
 * @param $post_num
 * @param array $args
 * @return \WP_Query
 */

function afz_get_related_posts_query($post_id, $post_num, $args = [])
{
    $terms = get_the_terms($post_id, 'category');

    if (empty($terms)) {
        $terms = [];
    }

    $term_list = wp_list_pluck($terms, 'slug');

    $related_args = [
        'post_type' => 'post',
        'posts_per_page' => $post_num,
        'post_status' => 'publish',
        'post__not_in' => [$post_id],
        'orderby' => 'rand',
        'tax_query' => [
            [
                'taxonomy' => 'category',
                'field' => 'slug',
                'terms' => $term_list,
            ],
        ],
    ];

    $related_args = array_merge($related_args, $args);

    return new WP_Query($related_args);
}

/**
 * 現在記事のカテゴリをカンマ区切りで取得する
 *
 * @return false|string
 */

function afz_get_the_comma_separated_categories()
{
    $categories = get_the_category();
    $category_text = '';

    foreach ($categories as $category) {
        $category_text .= $category->name.', ';
    }

    if (count($categories) > 0) {
        $category_text = substr($category_text, 0, -2);
    }

    return $category_text;
}

/**
 * 記事抜粋末尾の[...]を消す
 *
 * @param $text
 * @return string
 */

function afz_change_excerpt_end_text($text)
{
    return AFZ_EXCERPT_END_TEXT;
}

/**
 * 記事抜粋の文字数を制限
 *
 * @param $length
 * @return int
 */

function afz_change_excerpt_length($length)
{
    return AFZ_EXCERPT_LENGTH;
}

/**
 * ランキングのショートコード
 *
 * @param $attributes
 * @param null $content
 * @return string
 */

function afz_shortcode_ranking($attributes, $content = null)
{
    $attributes = shortcode_atts([
        'number' => '',
        'color' => '',
    ], $attributes, 'ranking');

    return '<div class="afz_ranking_title"><span class="afz_number '.$attributes['color'].'">'.$attributes['number'].'位</span>'.$content.'</div>';
}

/**
 * 吹き出しのショートコード
 *
 * @param $attributes
 * @param null $content
 * @return string
 */

function afz_shortcode_balloon($attributes, $content = null)
{
    $attributes = shortcode_atts([
        'facing' => 'left',
    ], $attributes, 'balloon');

    return '<div class="afz_balloon '.$attributes['facing'].'"><p>'.$content.'</p></div>';
}

/**
 * アイコン & 吹き出しのショートコード (左)
 *
 * @param $attributes
 * @param null $content
 * @return string
 */

function afz_shortcode_icon_balloon_left($attributes, $content = null)
{
    $attributes = shortcode_atts([
        'img' => '',
        'name' => '',
    ], $attributes, 'icon_balloon_left');

    return '<div class="afz_icon_balloon row"><div class="col-3 col-sm-2"><div class="afz_circle_img"><img src="'.$attributes['img'].'" alt=""></div><span class="afz_name">'.$attributes['name'].'</span></div><div class="afz_balloon left col-9 col-sm-10">
<p>'.$content.'</p></div></div>';
}

/**
 * アイコン & 吹き出しのショートコード (右)
 *
 * @param $attributes
 * @param null $content
 * @return string
 */

function afz_shortcode_icon_balloon_right($attributes, $content = null)
{
    $attributes = shortcode_atts([
        'img' => '',
        'name' => '',
    ], $attributes, 'icon_balloon_right');

    return '<div class="afz_icon_balloon right row"><div class="afz_balloon right col-9 col-sm-10"><p>'.$content.'</p></div><div class="col-3 col-sm-2"><div class="afz_circle_img"><img src="'.$attributes['img'].'" alt=""></div><span class="afz_name">'.$attributes['name'].'</span></div></div>';
}

/**
 * マーカーのショートコード
 *
 * @param $attributes
 * @param null $content
 * @return string
 */
function afz_short_code_marker($attributes, $content = null)
{
    $attributes = shortcode_atts([
        'color' => '',
    ], $attributes, 'marker');

    return '<span class="afz_marker marker_'.$attributes['color'].'"><strong>'.$content.'</strong></span>';
}

/**
 * ボックスのショートコード
 *
 * @param $attributes
 * @param null $content
 * @return string
 */

function afz_short_code_box($attributes, $content = null)
{
    $content = do_shortcode($content);

    return '<div class="afz_box">'.$content.'</div>';
}