<?php

class Toc_Shortcode
{
    private $add_script = false;

    private $attributes = [];

    public function __construct()
    {
        add_action('wp_enqueue_scripts', [$this, 'enqueue_scripts']);
        add_action('wp_footer', [$this, 'add_script'], 999999);
        add_filter('the_content', [$this, 'change_content'], 9);
        add_shortcode('toc', [$this, 'shortcode_content']);
    }

    function enqueue_scripts()
    {
        if (! wp_script_is('jquery', 'done')) {
            wp_enqueue_script('jquery');
        }
    }

    function change_content($content)
    {
        return "<div id=\"afz_toc_content\">{$content}</div>";
    }

    public function shortcode_content($attributes)
    {
        global $post;

        if (! isset($post)) {
            return '';
        }

        $this->attributes = shortcode_atts([
            'id' => '',
            'class' => 'afz_toc',
            'title' => '目次',
            'toggle' => true,
            'opentext' => '開く',
            'closetext' => '閉じる',
            'close' => false,
            'showcount' => 2,
            'depth' => 0,
            'toplevel' => 2,
            'offset' => '',
            'duration' => 'normal',
        ], $attributes);

        $this->attributes['toggle'] = (false !== $this->attributes['toggle'] && 'false' !== $this->attributes['toggle']) ? true : false;
        $this->attributes['close'] = (false !== $this->attributes['close'] && 'false' !== $this->attributes['close']) ? true : false;

        $content = $post->post_content;

        $headers = [];
        preg_match_all('/<([hH][1-6]).*?>(.*?)<\/[hH][1-6].*?>/u', $content, $headers);
        $header_count = count($headers[0]);
        $counter = 0;
        $counters = [0, 0, 0, 0, 0, 0];
        $current_depth = 0;
        $prev_depth = 0;
        $top_level = intval($this->attributes['toplevel']);
        if ($top_level < 1) {
            $top_level = 1;
        }
        if ($top_level > 6) {
            $top_level = 6;
        }
        $this->attributes['toplevel'] = $top_level;

        // 表示する階層数
        $max_depth = (($this->attributes['depth'] == 0) ? 6 : intval($this->attributes['depth']));

        $toc_list = '';
        for ($i = 0; $i < $header_count; $i++) {
            $depth = 0;
            switch (strtolower($headers[1][$i])) {
                case 'h1':
                    $depth = 1 - $top_level + 1;
                    break;
                case 'h2':
                    $depth = 2 - $top_level + 1;
                    break;
                case 'h3':
                    $depth = 3 - $top_level + 1;
                    break;
                case 'h4':
                    $depth = 4 - $top_level + 1;
                    break;
                case 'h5':
                    $depth = 5 - $top_level + 1;
                    break;
                case 'h6':
                    $depth = 6 - $top_level + 1;
                    break;
            }
            if ($depth >= 1 && $depth <= $max_depth) {
                if ($current_depth == $depth) {
                    $toc_list .= '</li>';
                }
                while ($current_depth > $depth) {
                    $toc_list .= '</li></ul>';
                    $current_depth--;
                    $counters[$current_depth] = 0;
                }
                if ($current_depth != $prev_depth) {
                    $toc_list .= '</li>';
                }
                if ($current_depth < $depth) {
                    $class = $current_depth == 0 ? ' class="toc-list"' : '';
                    $style = $current_depth == 0 && $this->attributes['close'] ? ' style="display: none;"' : '';
                    $toc_list .= "<ul{$class}{$style}>";
                    $current_depth++;
                }
                $counters[$current_depth - 1]++;
                $number = $counters[0];
                for ($j = 1; $j < $current_depth; $j++) {
                    $number .= '.'.$counters[$j];
                }
                $counter++;
                $toc_list .= '<li><a href="#toc'.($i + 1).'"><span class="afz_content_table_number">'.$number.'</span> '.$headers[2][$i].'</a>';
                $prev_depth = $depth;
            }
        }
        while ($current_depth >= 1) {
            $toc_list .= '</li></ul>';
            $current_depth--;
        }

        $html = '';
        if ($counter >= $this->attributes['showcount']) {
            $this->add_script = true;

            $toggle = '';
            if ($this->attributes['toggle']) {
                $toggle = ' <span class="toc-toggle">[<a class="internal" href="javascript:void(0);">'.($this->attributes['close'] ? $this->attributes['opentext'] : $this->attributes['closetext']).'</a>]</span>';
            }

            $html .= '<div'.($this->attributes['id'] != '' ? ' id="'.$this->attributes['id'].'"' : '').' class="'.$this->attributes['class'].'">';
            $html .= '<p class="afz_toc_title">'.$this->attributes['title'].$toggle.'</p>';
            $html .= $toc_list;
            $html .= '</div>'."\n";
        }

        return $html;
    }

    public function add_script()
    {
        if (! $this->add_script) {
            return false;
        }

        $class = $this->attributes['class'];
        $offset = is_numeric($this->attributes['offset']) ? (int) $this->attributes['offset'] : -1;
        $duration = is_numeric($this->attributes['duration']) ? (int) $this->attributes['duration'] : '"'.$this->attributes['duration'].'"';
        $opentext = $this->attributes['opentext'];
        $closetext = $this->attributes['closetext'];
        ?>
        <script type="text/javascript">
            (function ($) {
                var offset = <?php echo $offset; ?>;
                var idCounter = 0;
                $("#toc_content :header").each(function () {
                    idCounter++;
                    this.id = "toc" + idCounter;
                });
                $(".<?php echo $class; ?> a[href^='#']").click(function () {
                    var href = $(this).attr("href");
                    var target = $(href === "#" || href === "" ? "html" : href);
                    var h = (offset === -1 ? $("#wpadminbar").height() + $(".navbar-fixed-top").height() : offset);
                    var position = target.offset().top - h - 4;
                    $("html, body").animate({scrollTop: position}, <?php echo $duration; ?>, "swing");
                    return false;
                });
                $(".toc-toggle a").click(function () {
                    var tocList = $(this).parents(".<?php echo $class; ?>").children(".toc-list");
                    if (tocList.is(":hidden")) {
                        tocList.show("fast");
                        $(this).text("<?php echo $closetext; ?>");
                    } else {
                        tocList.hide("fast");
                        $(this).text("<?php echo $opentext; ?>");
                    }
                });
            })(jQuery);
        </script>
        <?php
    }
}